#!/usr/bin/python -tt
# -*- coding: utf-8 -*-
###########################################################
# Copyright 2014 Florian PERVES
# 
# This file is part of PYNBM.
# 
# PYNBM is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version
# 3 of the License, or (at your option) any later version.
# 
# PYNBM is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
# the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with PYNBM. If not, see
# http://www.gnu.org/licenses/.
###########################################################

import wx.lib.agw.aui as aui
import time
import wx
import os

class pynbm_nb(aui.AuiNotebook):
    def __init__(self,parent):
        self.tabgrids = []
        self.filegrids = {}
        self.activePageId = -1
        self.parent=parent
        aui.AuiNotebook.__init__(self,parent,agwStyle=aui.AUI_NB_DEFAULT_STYLE | aui.AUI_NB_TAB_SPLIT | aui.AUI_NB_CLOSE_ON_ALL_TABS | 
                                                      aui.AUI_NB_SMART_TABS)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.closeTab)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGED, self.OnPageChange)
        self.Bind(aui.EVT_AUINOTEBOOK_TAB_RIGHT_DOWN, self.OnRightClick)
        
        self.menu = wx.Menu()
        addTabMenu = wx.NewId()
        closeTabMenu = wx.NewId()
        self.menu.Append(addTabMenu, "Add Tab")
        self.menu.Append(closeTabMenu,"Close Tab")
        self.Bind(wx.EVT_MENU, self.addTab, id=addTabMenu)
        self.Bind(wx.EVT_MENU, self.closeTab, id=closeTabMenu)
        return
    
    def addTab(self,evt):
        self.parent.on_newtab(None)

    def closeAllTabs(self):
        cancel=False
        while (len(self.tabgrids) > 0):
#            print "*******************************"
            if self.OnPageClose(None)=="Cancel":
                cancel=True
                break
        return cancel

    def closeTab(self,evt):
        try:
            if self.OnPageClose(evt)=="Cancel":
                pass
            else:
                filepath=self.filegrids[self.GetCurrentPage()]
                self.parent.removeFromLastOpenedFiles(filepath)
        except:
            pass
        
        


        
    
    def OnPageChange(self, evt):
        page = self.GetCurrentPage()
        try:
            filepath=self.parent.version + " - " + str(self.filegrids[page])
            filename="File : " + os.path.basename(self.filegrids[page])
        except:
            filepath=self.parent.version
            filename="No file loaded in this tab"
        self.parent.SetTitle(filepath)
        self.parent.bar.SetStatusText(filename,1)

#        print str(self.tabgrids)
#        print "current : " + str(self.GetCurrentPage())

    def OnPageClose(self, evt):
        page = self.GetCurrentPage()
#        print "tabgrids : " + str(self.tabgrids)
#        print "currentSelection : " + str(self.GetSelection())
        
        if self.tabgrids[self.GetSelection()].modifiedFlag==True:
            dlg = wx.MessageDialog(self, 
                              u"Do you want to save the grid before closing ?",
                              u"Save the grid before closing ?" ,
                              wx.YES_NO | wx.ICON_INFORMATION | wx.CANCEL
    #wx.YES_NO | wx.NO_DEFAULT | wx.CANCEL | wx.ICON_INFORMATION
                          )
            dlgreturn = dlg.ShowModal()
            dlg.Destroy()
            if dlgreturn == wx.ID_CANCEL:
                evt.Veto()
                return "Cancel"
            if dlgreturn == wx.ID_YES:
                save = self.parent.on_export(self)   
            # wait for refresh threads for this tab to finish before closing the tab
        while self.parent.GetRunningThreadsForGrid(page)!=0:
            time.sleep(1)
        self.tabgrids.remove(self.GetCurrentPage())
        if evt==None:
#            print "self.parent.getActiveTabId() : " + str(self.GetSelection())
#            print "self.GetCurrentPage() : " + str(self.GetCurrentPage())
 

            self.DeletePage(self.GetSelection())
#        print "fin du pageclose"
        
    def OnRightClick(self,evt):

        self.PopupMenu(self.menu)